/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002-2003 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package moka.io;

import moka.io.IOStream;
import moka.lang.Character;

/**
 * An abstract representation of file and directory pathnames.
 *
 * @author  Frdric Brown
 * @version 1.1, 2003-11-02
 * @since   MDK1.0a
 */
public class File extends IOStream {

	/** Open for reading only (binary).*/
	public static final short MODE_OPEN_READ = 0;
	/** Create for writing (binary). If a file by that name already exists, it will be overwritten.*/
	public static final short MODE_WRITE = 1;
	/** Append (binary); open for writing at end of file, or create for writing if the file does not exist.*/
	public static final short MODE_APPEND = 2;
	/** Open an existing file for update (reading and writing) (binary).*/
	public static final short MODE_OPEN_UPDATE = 3;
	/** Create a new file for update (reading and writing) (binary). If a file by that name already exists, it will be overwritten.*/
	public static final short MODE_UPDATE = 4;
	/** Open for append (binary); open for update at the end of the file, or create if the file does not exist.*/
	public static final short MODE_UPDATE_APPEND = 5;

	/** Open for reading only (text).*/
	public static final short MODE_OPEN_READ_TEXT = 6;
	/** Create for writing (text). If a file by that name already exists, it will be overwritten.*/
	public static final short MODE_WRITE_TEXT = 7;
	/** Append (text); open for writing at end of file, or create for writing if the file does not exist.*/
	public static final short MODE_APPEND_TEXT = 8;
	/** Open an existing file for update (reading and writing) (text).*/
	public static final short MODE_OPEN_UPDATE_TEXT = 9;
	/** Create a new file for update (reading and writing) (text). If a file by that name already exists, it will be overwritten.*/
	public static final short MODE_UPDATE_TEXT = 10;
	/** Open for append (text); open for update at the end of the file, or create if the file does not exist.*/
	public static final short MODE_UPDATE_APPEND_TEXT = 11;

	/** The FILE structure encapsulated. */

	/** The file mode. Should be only read.*/
	public short mode;

	/** The name of the file. Should be only read.*/
	public String name = null;

	/** The type of the file to appear in the VAR-LINK menu.*/
	public String type = null;

    /**
     * Creates a new <code>File</code> instance by using the given pathname string.
     * If the pathname string object risk to be used later, use a copy of the string object
     * instead of the original.
     *
     @param pathname A pathname string
     */
	public File (String pathname) {

	}
	
	/**
	 * Shows the VAR-LINK dialog and return a file object, associated to the TIOS variable selected in the dialog.
	 * Using this method will force the main class of a project to implement the No_ROMCallsOptimization interface (otherwise the program would crash).
	 *
	 * @return a file object associated to the TIOS variable selected in the dialog, or null if no file was selected.
	 */
	public static native File showVarLinkDialog() ;
	
	/**
	 * Exports the long argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * l The long to export
	 */
	public static void export (String name, long l) {
		File.exportExpr(name, String.valueOf(l));
	}

	/**
	 * Exports the int argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * i The int to export
	 */
	public static void export (String name, int i) {
		File.exportExpr(name, String.valueOf(i));
	}
	
	/**
	 * Exports the short argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * s The short to export
	 */
	public static void export (String name, short s) {
		File.exportExpr(name, String.valueOf(s));
	}
	
	/**
	 * Exports the byte argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * b The byte to export
	 */
	public static void export (String name, byte b) {		
		File.exportExpr(name, String.valueOf(b));
	}
	
	/**
	 * Exports the double argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * d The double to export
	 */
	public static void export (String name, double d) {		
		File.exportExpr(name, String.valueOf(d));
	}
	
	/**
	 * Exports the char argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * c The char to export
	 */
	public static void export (String name, char c) {		
		File.export(name, String.valueOf(c));
	}
	
	/**
	 * Exports the boolean argument to a TIOS variable.
	 * If the name string object risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * b The boolean to export
	 */
	public static void export (String name, boolean b) {		
		File.export(name, String.valueOf(b));
	}
	
	/**
	 * Exports the String argument to a TIOS variable.
	 * If the name and s string objects risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * s The String to export
	 */
	public static void export (String name, String s) {
		String str = "\"";
		
		str += s;
		
		str += "\"";
		
		File.exportExpr(name, str);
	}
	
	/**
	 * Exports an expression to a TIOS variable.
	 * If the name and expr string objects risk to be used later, use a copy of the string object
     * instead of the original.
	 *
	 * name The name of the TIOS variable
	 * expr The String to export
	 */
	public static void exportExpr (String name, String expr) {	
		//expr.append((char)22);
		
		expr += ((char)22);
		
		expr += name;
		
		System.exec(expr);
	}

	/**
	 * Opens an existing binary file in the current I/O mode (read only by default).
	 */
	public void open() {

	}

	/**
	 * Opens a binary file.
	 @param mode The file mode
	 */
	public native void open(short mode) ;

	/**
	 * Closes the stream.
	 */
	public void close() {

	}

	/**
	 * Returns the number of bytes that can be read from this file.
	 *
	 * @return	the number of bytes that can be read from this file.
	 */
	public native int available () ;

    /**
     * Deletes the file denoted by this abstract pathname.
     *
     * @return  <code>true</code> if and only if the file is
     *          successfully deleted; <code>false</code> otherwise
     */
    public native boolean delete() ;

	/**
	 * Tests whether the file denoted by this abstract pathname exists.
	 *
     * @return  <code>true</code> if and only if the file denoted by this
     *          abstract pathname exists; <code>false</code> otherwise
	*/
	public native boolean exists () ;

	/**
	 * Tests whether the file named by this abstract pathname is archived.
	 *
	 * @return true if and only if the file denoted by this abstract pathname is archived
	*/
	public native boolean isArchived () ;

	/**
	 * Tests whether the file named by this abstract pathname is hidden.
	 *
	 * @return true if and only if the file denoted by this abstract pathname is hidden
	*/
	public native boolean isHidden () ;
	/**
	 * Tests whether the file named by this abstract pathname is locked.
	 *
	 * @return true if and only if the file denoted by this abstract pathname is locked
	*/
	public native boolean isLocked () ;

    /**
     * Returns the length of the file denoted by this abstract pathname.
     *
     * @return  The length, in bytes, of the file denoted by this abstract
     *          pathname
     */
    public int length() {

		return 0;
    }

    /**
     * Creates the directory named by this abstract pathname.
     *
     * @return  <code>true</code> if and only if the directory was
     *          created; <code>false</code> otherwise
     */
    public native boolean mkdir() ;
    /**
     * Renames the file denoted by this abstract pathname.
     * If the dest string object risk to be used later, use a copy of the string object
     * instead of the original.
     *
     * @param  dest  The new abstract pathname for the named file
     *
     * @return  <code>true</code> if and only if the renaming succeeded;
     *          <code>false</code> otherwise
     */
    public native boolean renameTo(String dest) ;

    /**
     * Removes the directory named by this abstract pathname including
     * all files in it.
     *
     * @return  <code>true</code> if and only if the directory was
     *          removed; <code>false</code> otherwise
     */
    public native boolean rmdir() ;

	/**
	 * Sets the file-pointer offset, measured from the beginning of this file, at which the next read or write occurs.
	 *
	 * @param pos the offset position, measured in bytes from the beginning of the file, at which to set the file pointer.
	 */
	public native void seek (int pos) ;
	/**
	 * Sets if the file named by this abstract pathname is archived (in the archive memory instead of the RAM).
	 @param b a flag indicating whether this file component is archived.
	 @return true if and only if the operation succeeded; false otherwise
	*/
	public native boolean setArchived (boolean b) ;

	/**
	 * Sets if the file named by this abstract pathname is hidden (invisible for the TI-BASIC programs).
	 @param b a flag indicating whether this file is Hidden.
	 @return true if and only if the operation succeeded; false otherwise
	*/
	public native boolean setHidden (boolean b) ;
	/**
	 * Sets if the file named by this abstract pathname is locked (cannot be altered).
	 @param b a flag indicating whether this file is locked.
	 @return true if and only if the operation succeeded; false otherwise
	*/
	public native boolean setLocked (boolean b) ;
	/**
	 * Skips over n bytes of data from this file.
	 * @param n the number of bytes to be skipped.
	 */
	public native void skip (int n) ;

    /**
     * Writes an 8 bit byte.
     *
     * @param	val the byte value to be written
     */
	public native void writeByte(char val) ;

    /**
     * Writes a buffer of bytes up to the specified length.
     *
     * @param	buffer the buffer of bytes to be written
     * @param	len the specified length
     */
	public native void writeBytes(char_p buffer, short len) ;

    /**
     * Reads an 8 bit byte.
     *
     * @return	the byte value read
     */
	public char readByte() {

		return ' ';
	}

    /**
     * Copies to a buffer of bytes up to the specified length bytes read from the stream.
     *
     * @param	buffer the buffer of bytes where the bytes will be copied
     * @param	len the specified length
     */
	public native void readBytes(char_p buffer, short len) ;

    /**
     * Frees the memory of system ressources used by this File
     * object.  If the stream is opened, it will be closed first.
     */
	public void finalize() {
	}
}