/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package moka.io;

import moka.lang.System;
import moka.lang.Character;

/**
 * The Graphics class encapsulates the display device operations
 * and provides methods to deal with bitmaps. Note that some methods has a static version. These methods are written with
 * an uppercase letter in this document to differenciate them from their instance counterparts, but they must be write with
 * a lowercase in a program. Ex: Graphics.drawTo(10, 10) (Althrought it is written DrawTo in this doccument)
 *
 * @author  Frdric Brown
 * @version 1.1, 2003-10-25
 * @since   MDK1.0a
 */
public class Graphics extends moka.lang.Object {
	/** The graphic port address. LCD (addr: 0x4C00,) by default.*/
	public void_p addr;

	/** The maximum x coordinate.*/
	public short x_max;

	/** The maximum y coordinate.*/
	public short y_max;

	/** Draw with destination pixels turned off.*/
	public static final short REVERSE_ATTR = 0;

	/** Draw with destination pixels turned on.*/
	public static final short NORMAL_ATTR = 1;

	/** Draw with source pixels XORed with destination pixels.*/
	public static final short XOR_ATTR = 2;

	/** Draw with destination pixels masked so that every other pixel turned off*/
	public static final short SHADED_ATTR = 3;

	/** Draw with source pixels replace destination pixels.*/
	public static final short REPLACE_ATTR = 4;

	/** Draw with source pixels ORed with destination pixels.*/
	public static final short OR_ATTR = 5;

	/** AND the region to draw.*/
	public static final short AND_ATTR = 00;

	/** Draw a double thick line.*/
	public static final short THICK1_ATTR = 6;

	/** Draw the line using a vertical shading pattern.*/
	public static final short SHADE_V_ATTR = 7;

 	/** Draw the line using a horizontal shading pattern.*/
	public static final short SHADE_H_ATTR = 8;

	/** Draw the line using a negative slope diagonal shading pattern.*/
	public static final short SHADE_NS_ATTR = 9;

	/** Draw the line using a positive slope diagonal shading pattern.*/
	public static final short SHADE_PS_ATTR = 10;

	/** A universal SCR_RECT.*/
	public static SCR_RECT_p uScrRect;

	/** The actual grayscale level (the gray variant the graphic instructions will draw). Legal value are Gray.OFF(grayscale disabled), Gray.BLACK, Gray.LIGHT_GRAY and Gray.DARK_GRAY.*/
	public static short level = Gray.OFF;
	
	/** Is in grayscale mode ? Should be only read*/
	public static boolean graymode = false;

	/**
	 * Creates a new Graphics object using the default display device.
	 */
	public Graphics () {
	}

	/**
	 * Creates a new Graphics object specifying the display device.
	 *
	 * @param addr The graphic port address
	 * @param x_max The maximum x coordinate
	 * @param y_max The maximum y coordinate
	 */
	public Graphics (void_p addr, short x_max, short y_max) {
	}

	/**
	 * Sets the grayscale level.
	 * 
	 * @param g the grayscale level. Legal values are : Gray.OFF(grayscale disabled), Gray.BLACK, Gray.LIGHT_GRAY and Gray.DARK_GRAY.
	 */
	public static native void setGray(short g);

	/**
	 * Clears the current display device screen.  The pen
	 * position is moved to (0, 0). The main difference of this
	 * method vs the System class' one is its grayscale support.
	 */
	public static void clrscr () {
		Graphics.moveTo(0, 0);
		Graphics.clrScr();
	}

	/**
	 * Clears the Graphics' display device screen.  The pen
	 * position will remain intact.
	 * Changes the current graphic port to the one of the graphic object.
	 */
	public native void clrScr () ;

	/**
	 * Draws a char on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param c The char to draw
	 * @param attr The attribute
	 */
	public native void drawChar (short x, short y, char c, short attr) ;

	/**
	 * Draws a pixel on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.XOR_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param attr The attribute
	 */
	public native void drawPix (short x, short y, short attr) ;

	/**
	 * Draws a line on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.XOR_ATTR, Graphics.THICK1_ATTR, Graphics.SHADE_V_ATTR, Graphics.SHADE_H_ATTR,
	 * Graphics.SLOPE_NS_ATTR, Graphics.SLOPE_PS_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x0 The x coordinate of the upper left corner
	 * @param y0 The y coordinate of the upper left corner
	 * @param x1 The x coordinate of the lower right corner
	 * @param y1 The y coordinate of the lower right corner
	 * @param attr The attribute
	 */
	public native void drawLine (short x0, short y0, short x1, short y1, short attr) ;

	/**
	 * Draws an ANSI string on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param str The ANSI string to draw
	 * @param attr The attribute
	 */
	public native void draw (short x, short y, char_p str, short attr) ;

	/**
	 * Draws a String object on the Graphics' display device screen then finalizes the String object.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param str The String to draw
	 * @param attr The attribute
	 */
	public native void draw (short x, short y, String str, short attr) ;

	/**
	 * Draws a short on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The short to draw
	 * @param attr The attribute
	 */
	public void draw (short x, short y, short num, short attr) ;

	/**
	 * Draws an int on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The int to draw
	 * @param attr The attribute
	 */
	public void draw (short x, short y, int num, short attr) {

	}

	/**
	 * Draws a long on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The long to draw
	 * @param attr The attribute
	 */
	public void draw (short x, short y, long num, short attr) {

	}

	/**
	 * Draws a double on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The double to draw
	 * @param attr The attribute
	 */
	public void draw (short x, short y, double num, short attr) {

	}

	/**
	 * Draws a char on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param c The char to draw
	 * @param attr The attribute
	 */
	public void draw (short x, short y, char c, short attr) {

	}

	/**
	 * Gets the status of a pixel on the Graphics' display device screen.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @return true if the pixel is on, false otherwize
	 */
	public native boolean getPix (short x, short y) ;

	/**
	 * Sets the virtual screen to the Graphics' display device screen.  All graphic commands which are built-in into TIOS
	 * does not expect that the video memory must be at 0x4C00, and that the video memory
	 * is always 240 x 128 pixels. Using portSet you can set up a virtual screen anywhere in
	 * a memory, and of any size.
	 */
	public native void portSet () ;

	/**
	 * Draws a elipse on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.XOR_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate of the elipse
	 * @param y The y coordinate of the elipse
	 * @param a The horizontal semiaxe of the elipse
	 * @param b The vertical semiaxe of the elipse
	 * @param attr The attribute
	 */
	public native void drawElipse (short x, short y, short a, short b, short attr) ;

	/**
	 * Draws a line from the current pen position (set by moveTo) on the Graphics' display device screen.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 */
	public native void drawTo (short x, short y) ;

	/**
	 * Saves the content of the LCD screen to a new LCD_BUFFER.
	 *
	 * @return A pointer to a newly allocated LCD_BUFFER.
	 */
	public native static void_p lcdSave () ;

	/**
	 * Saves the content of the LCD screen to an existing LCD_BUFFER.
	 *
	 * @param buffer The LCD_BUFFER
	 * @return A pointer to the LCD_BUFFER.
	 */
	public native static void_p lcdSave (void_p buffer) ;

	/**
	 * Restores the saved content of the LCD screen.
	 *
	 * @param buffer The saved LCD_BUFFER
	 */
	public native static void lcdRestore (void_p buffer) ;

	/**
	 * Sets the virtual screen.  All graphic commands which are built-in into TIOS
	 * does not expect that the video memory must be at 0x4C00, and that the video memory
	 * is always 240 x 128 pixels. Using portSet you can set up a virtual screen anywhere in
	 * a memory, and of any size.
	 *
	 * @param addr The memory address
	 * @param x_max The maximum x coordinate
	 * @param y_max The maximum x coordinate
	 */
	public native static void portSet (void_p addr, short x_max, short y_max) ;

	/**
	 * Clears the current display device screen.  The pen
	 * position will remain intact.
	 */
	public native static void ClrScr () ;

	/**
	 * Draws a char on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param c The char to draw
	 * @param attr The attribute
	 */
	public native static void DrawChar (short x, short y, char c, short attr) ;

	/**
	 * Draws a pixel on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.XOR_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param attr The attribute
	 */
	public native static void DrawPix (short x, short y, short attr) ;

	/**
	 * Draws a line on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.XOR_ATTR, Graphics.THICK1_ATTR, Graphics.SHADE_V_ATTR, Graphics.SHADE_H_ATTR,
	 * Graphics.SLOPE_NS_ATTR, Graphics.SLOPE_PS_ATTR.
	 *
	 * @param x0 The x coordinate of the upper left corner
	 * @param y0 The y coordinate of the upper left corner
	 * @param x1 The x coordinate of the lower right corner
	 * @param y1 The y coordinate of the lower right corner
	 * @param attr The attribute
	 */
	public native static void DrawLine (short x0, short y0, short x1, short y1, short attr) ;

	/**
	 * Draws a ellipse on the Graphics' display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.XOR_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate of the ellipse
	 * @param y The y coordinate of the ellipse
	 * @param a The horizontal semiaxe of the ellipse
	 * @param b The vertical semiaxe of the ellipse
	 * @param attr The attribute
	 */
	public native void drawEllipse (short x, short y, short a, short b, short attr) ;

	/**
	 * Gets the status of a pixel on the current display device screen. Intented to
	 * be used while in grayscale mode.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @return the status of the pixel (Gray.OFF if grayscale is turned off or the pixel is off, grayscale level otherwize).
	 */
	/*Alias implemented for getPix (short x, short y).*/
	public native static short getPixGray (short x, short y) ;

	/**
	 * Draws a line from the current pen position (set by moveTo) on the current display device screen.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 */
	public native static void DrawTo (short x, short y) ;

	/**
	 * Move the current pen position.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 */
	public native static void MoveTo (short x, short y) ;

	/**
	 * Draws an ANSI string on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param str The ANSI string to draw
	 * @param attr The attribute
	 */
	public native static void Draw (short x, short y, char_p str, short attr) ;

	/**
	 * Draws a String object on the current display device screen then finalizes the String object.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param str The String to draw
	 * @param attr The attribute
	 */
	public native static void Draw (short x, short y, String str, short attr) ;

	/**
	 * Draws a short on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The short to draw
	 * @param attr The attribute
	 */
	public static void Draw (short x, short y, short num, short attr) {

	}

	/**
	 * Draws an int on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The int to draw
	 * @param attr The attribute
	 */
	public static void Draw (short x, short y, int num, short attr) {

	}

	/**
	 * Draws a long on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The long to draw
	 * @param attr The attribute
	 */
	public static void Draw (short x, short y, long num, short attr) {

	}

	/**
	 * Draws a double on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param num The double to draw
	 * @param attr The attribute
	 */
	public static void Draw (short x, short y, double num, short attr) {

	}

	/**
	 * Draws a char on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @param c The char to draw
	 * @param attr The attribute
	 */
	public static void Draw (short x, short y, char c, short attr) {

	}

	/**
	 * Draws an ellipse on the current display device screen.
	 * The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR,
	 * Graphics.XOR_ATTR.
	 * Changes the current graphic port to the one of the graphic object.
	 *
	 * @param x The x coordinate of the ellipse
	 * @param y The y coordinate of the ellipse
	 * @param a The horizontal semiaxe of the ellipse
	 * @param b The vertical semiaxe of the ellipse
	 * @param attr The attribute
	 * @since   MDK2.1
	 */
	public static native void DrawEllipse (short x, short y, short a, short b, short attr) ;

	/**
	 * Gets the status of a pixel on the current display device screen.
	 *
	 * @param x The x coordinate
	 * @param y The y coordinate
	 * @return true if the pixel is on, false otherwize
	 */
	public native static boolean GetPix (short x, short y) ;

	/**
	 * Converts a specified PIC file name to a bitmap.
	 * If the file string object risk to be used later, use a copy of the string object
     * instead of the original, the string being finalized before the method returns.
	 *
	 * @param file The PIC file name
	 * @return The converted bitmap or null in case of error.
	 */
	public native static BITMAP_p bitmapFromVar(String file) ;

	/**
	 * Saves a specified bitmap to a specified pic file.
	 * If the file string object risk to be used later, use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
	 *
	 * @param bmp The bitmap
	 * @param file The PIC file name
	 */
	public native static void bitmapToVar(BITMAP_p bmp, String file) ;

	/**
	 * Return a bitmap, taken from a specified zone of the screen.
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param width The width of the zone
	 * @param height The height of the zone
	 */
	public static BITMAP_p getBitmap (short x, short y, short width, short height) { return null;}

	/**
	 * Saves a zone of the screen to a TIOS' PIC variable.
	 * If the file string object risk to be used later, use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
	 * @param file The PIC file name
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param width The width of the zone
	 * @param height The height of the zone
	 */
	public static void getPicVar (String file, short x, short y, short width, short height) {
	}
	
	/**
	 * Puts a TIOS PIC variable on the screen. The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR, Graphics.OR_ATTR, Graphics.AND_ATTR. 
	 * If the file string object risk to be used later, use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
	 * @param file The PIC file name
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param attr Display attribute
	 */
	public static void picVarPut (String file, short x, short y, short attr) {}

	/**
	 * Puts a TIOS PIC variable on the screen. The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR, Graphics.OR_ATTR, Graphics.AND_ATTR. 
	 * @param SymName TIOS variable name, in vat string format
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param attr Display attribute
	 */
	public native static void picVarPut (char_p SymName, short x, short y, short attr) ;

	/**
	 * Returns a copy of the specified bitmap.
	 *
	 * @param bmp The bitmap
	 * @return A copy of the specified bitmap
	 */
	public native static BITMAP_p cloneBitmap(BITMAP_p bmp) ;
	
	/**
	 * Puts a BITMAP on the screen. The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR, Graphics.OR_ATTR, Graphics.AND_ATTR.
	 * @param b The BITMAP
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param attr Display attribute
	 */
	public native static void putBitmap (BITMAP_p b, short x, short y, short attr) ;

	/**
	 * Puts a set of TIOS on the screen. The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR, Graphics.OR_ATTR, Graphics.AND_ATTR.
	 * Be warned that this method will enable grayscale.
	 * @param d The BITMAP of the dark plane (layer 1)
	 * @param l The BITMAP of the light plane (layer 2)
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param attr Display attribute
	 */
	public native static void putGray (BITMAP_p d, BITMAP_p l, short x, short y, short attr) ;

	/**
	 * Puts a set of TIOS PIC variables on the screen. The following attributes are supported: Graphics.NORMAL_ATTR, Graphics.REVERSE_ATTR, Graphics.REPLACE_ATTR, Graphics.XOR_ATTR, Graphics.SHADED_ATTR, Graphics.OR_ATTR, Graphics.AND_ATTR.
	 * Be warned that this method will enable grayscale.
	 * @param d The PIC file name of the dark plane (layer 1)
	 * @param l The PIC file name of the light plane (layer 2)
	 * @param x Absolute horizontal coordinate
	 * @param y Absolute vertical coordinate
	 * @param attr Display attribute
	 */
	public static void putGray (String d, String l, short x, short y, short attr) {
	}
	
}