/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package moka.x;

import moka.io.Keyboard;
import moka.x.Component;
import moka.x.Container;
import moka.x.Frame;
import moka.x.Font;
import moka.event.EventListener;
import moka.event.Interrupt;
import moka.util.Timer;

/**
 * The GEM (Graphic Environment Manager) class implements the methods used
 * to manage the Graphic Environment.
 *
 * @author  Frdric Brown
 * @version 1.11, 2002-10-30
 * @since   MDK1.0a
 */
public class GEM extends Container {

	/** The state of the lcd before x was started.*/
	private void_p lcd;

	/** Is an interrupt occured ?*/
	private boolean interupted;

	/** The array of interrupts.*/

	/** The number of interrupts.*/
	private int size;

	/** The actual interrupts' index.*/
	private int index;

	/** The capacity of the interrupts' array.*/
	private int capacity;

	/** A keyboard object used when moving the cursor.*/
	private Keyboard kb;

	/** The difference between the cursor horizontal coordinate and the dragged component horizontal coordinate.*/
	private short difX;

	/** The difference between the cursor vertical coordinate and the dragged component vertical coordinate.*/
	private short difY;

	/** The context of the dragged component.*/
	private SCR_RECT dragRect;

	/** The image behind the cursor.*/
	private void_p cursorImg;

	/** The image of the component being dragged.*/
	private void_p dragImg;

	/** The virtual screen buffer.*/
	protected void_p port;

	/** The instance of the Graphic Environment Manager. Should be only read.*/
	public static GEM gem;

	/** Height of the screen in pixels (100 on TI-89, 128 on TI-92 Plus or V200).*/
	public static short LCD_HEIGHT;
	/** Number of bytes in the visible part of a screen line (20 on TI-89, 30 on TI-92 Plus or V200).*/
	public static short LCD_LINE_BYTES;
	/** Width of the screen in pixels (160 on TI-89, 240 on TI-92 Plus or V200).*/
	public static short LCD_WIDTH;
	/** The usable height of the screen (94 on TI-89, 122 on TI-92 Plus or V200)*/
	public static short USABLE_HEIGHT;
	/** The virtual screen SCR_RECT.*/
	public static SCR_RECT_p vScrRect;

	/** Is the Graphic Environment started ? Should be only read.*/
	public boolean started;

	/** The cursor x position. Should be only read.*/
	public short cursorX;
	/** The cursor y position. Should be only read.*/
	public short cursorY;
	/** Is the cursor visible ? Should be only read.*/
	public short cursorVisible;
	/** The horizontal speed of the cursor.*/
	public short cursorSpeedX;
	/** The vertical speed of the cursor.*/
	public short cursorSpeedY;

	/** The ssytem font.  Should be only read.*/
	public short systemFont;

	/** The event listener notified when the Graphic Environment starts.*/
	public EventListener startListen;

	/** The event listener notified when a frame is minimized.*/
	public EventListener minListen;

	/*
	 * Constructs a newly allocated Graphic Environment Manager.
	 * This constructor is called at the startup of the application
	 * to initialize GEM.gem (The system default Graphical Environment
	 * Manager).
	 */
	public GEM () {
	}

    /**
     * Free the memory and system ressources used by this GEM.
     */
	public native void finalize () ;

	/**
	 * Called when an UpdateEvent occurs.  An update event occurs when a
	 * visible component contained in this container has changed its
	 * appearance.
	 */
	public void onRefresh () {
	}

	/**
	 * Disposes of all ressources used by the Graphic Environment.
	 */
	protected void clean () {
	}

	/**
	 * Starts this Graphic Environment Manager as if setStarted(true) was called.
	 */
	/*public void startx () {
		this.setStarted(true);
	}*/

	/**
	 * Starts or stops this GEM. While the GEM is started, the execution flow become even driven instead of
	 * linear i.e. that instructions are executed when events or interrupts occur instead
	 * of being executed sequentially. When the GEM is stopped, the execution flow continues were it was.
	 *
	 * @param b true if the Graphic Environment should be started, false if the
	 *			Graphic Environment should be stopped.
	 */
	public void setStarted (boolean b) {
	}

	/**
	 * Called when a PaintEvent occurs to refresh the screen. All
	 * the components are painted.  A PaintEvent occurs when the
	 * aspect of any component is altered.
	 */
	public native void onPaint () ;

	/**
	 * Sets the visibility of the cursor.
	 *
	 * @param b true if the cursor should be visible, false otherwise.
	 */
	public native void setCursorVisible (boolean b) ;

	/**
	 * Gives the user the control of the cursor.
	 *
	 * @return The last key read from the keyboard.
	 */
	public short useCursor () {

		return 0;
	}

	/**
	 * Removes the specified component from the GEM.
	 *
	 * @param c the component to remove from the GEM.
	 */
	/*public void remove (Component c) {
		super.remove(c);
		if (!this.child) {
			this.setStarted(false);
		}
	}*/

	/**
	 * Registers the specified interrupt.
	 *
	 * @param item the interrupt to be registered.
	 */
	public native void register (Interrupt item) ;

	/**
	 * Unregisters the specified interrupt.
	 *
	 * @param item the interrupt to be unregistered.
	 */
	public native void unregister (Interrupt item) ;

	/**
	 * This method is looping when the GEM is started.
	 */
	public void run () {
	}

	/**
	 * This method is called when a Frame is showed in modal mode.
	 *
	 * @param frame The frame to show in modal mode.
	 */
	public void showModal (Frame frame) {
	}

	/**
	 * Called when a DragEvent occurs to drag and drop a component.
	 * A DragEvent occurs when a component needs to be moved by the
	 * user.
	 *
	 * @param sender the object who triggered the DragEvent.
	 */
	public void onDrag (Component sender) {
	}

	/**
	 * Sets the system font.
	 *
	 * @param font the new system font.
	 */
	public void setFont (short font) {
	}

	/**
	 * Returns the specified font height.
	 *
	 * @param font the specified font
	 * @return the specified font height
	 */
	public static short getFontHeight (short font) {
		return 0;
	}

	/**
	 * Returns the specified font width.
	 *
	 * @param font the specified font
	 * @return the specified font width
	 */
	public static short getFontWidth (short font) {

		return 0;
	}

	/**
	 * Returns the current system font height.
	 *
	 * @return the current system font height
	 */
	public short getFontHeight () {
		return 0;
	}

	/**
	 * Returns the current system font width.
	 *
	 * @return the current system font width
	 */
	public short getFontWidth () {
		return 0;
	}

	/**
	 * Called when a MinimizedEvent is triggered. A MinimizeEvent
	 * is triggered when a user wants to minimize a frame.
	 */
	public void onMinimize (Frame sender) {
	}
	
	/**
	 * Sets the current screen state as background.
	 *
	 * @since MDK2.11
	 */
	public native void setBackground();
}