/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package moka.x;

import moka.lang.Long;
import moka.lang.Double;
import moka.lang.Character;
import moka.x.CaptionedComponent;

/**
 * TextField class is a text entry component.
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-11-04
 * @since   MDK1.0a
 */
public class TextField extends CaptionedComponent {

	/** The TextField accepts any character.*/
	public static final short NO_INPUT_TYPE = 0;
	/** The TextField only accepts integers.*/
	public static final short INT_INPUT_TYPE = 1;
	/** The TextField only accepts integers and floats.*/
	public static final short FLOAT_INPUT_TYPE = 2;
	/** The TextField only accepts characters.*/
	public static final short CHAR_INPUT_TYPE = 3;

	/** The number of character this TextField should accept. Should be > 0.*/
	public short maxLength;

	/** The input type of this TextField*/
	public short inputType;

	/** The mask for this TextField. '\0' (default) to disable. Should be only read.*/
	public char mask;

	/*
	 * Constructs a new TextField.
	 */
	public TextField () {
		this("");
	}

	/*
	 * Constructs a new TextField specifying its caption.
	 *
	 * @param caption the TextField's caption.
	 */
	public TextField (String caption) {
		super(caption);
		this.maxLength = 255;
		this.inputType = TextField.NO_INPUT_TYPE;
		this.mask = 0;
	}

	/**
	 * Paints the textfield.
	 */
	public native void paint() ;

	/**
	 * This method is invoked by the GEM when the user interacts
	 * with the TextField.
	 */
	public void use () {
	}

    /**
     * Checks if the specified key is a valid input for the
     * input type of this TextField.
     *
     * @param key The key to verify.
     * @return true if the specified key is a valid input, false otherwize.
     */
    private boolean checkInput (short key, char_p buffer, short ind) {

    	return true;
    }
    
 	/**
 	 * Sets the mask of the TextField. Set to '\0' to disable mask.
 	 *
 	 * @param mask The mask of the TextField.
 	 */
	public void setMask (char mask) {
		this.mask = mask;
		
		this.repaint = true;
		if (this.visible && this.parent) {
			this.parent.onRefresh();
		}
	}
}