/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.io.Keyboard;//Import the class moka.io.Keyboard
import moka.io.Socket;//Import the class moka.io.Socket

/**
 * This class represents the remote player of the Tron game.  This class is a subclass of Player.
 *
 * @author  Frdric Brown
 */
public class NetPlayer extends Player {
	/** A socket object used to receive the direction of the remote player.*/
	private Socket sok;

	/**
	 * Initializes a newly allocated Player2.
	 */
	public NetPlayer () {
		super(2s);//Call the superclass' constructor, passing 2 as argument
		this.sok = new Socket();//Instanciate a new Socket
	}

	/**
	 * Reads the keyboard to set the new direction
	 * of the Player.  This method, in fact receive
	 * the direction from the remote player instead of
	 * reading the remote player's keyboard.
	 *
	 * @param kb The Keyboard to read
	 */
	public void readKb (Keyboard kb) {
			short dir;//A short used to contain the direction of the remote player
			
			this.sok.open();//Open the socket
			
			dir = this.sok.readShort();//Read a short from the socket
			
			if (!this.sok.error) {//If the socket is not in error
			
				if (dir == Player.UP_DIRECTION && this.direct != Player.DOWN_DIRECTION) {//If UP is pressed
					this.direct = Player.UP_DIRECTION;//Set the player 2 direction to up
				}
				else if (dir == Player.DOWN_DIRECTION && this.direct != Player.UP_DIRECTION) {//If DOWN is pressed
					this.direct = Player.DOWN_DIRECTION;//Set the player 2 direction to down
				}
				else if (dir == Player.LEFT_DIRECTION && this.direct != Player.RIGHT_DIRECTION) {//If LEFT is pressed
					this.direct = Player.LEFT_DIRECTION;//Set the player 2 direction to left
				}
				else if (dir == Player.RIGHT_DIRECTION && this.direct != Player.LEFT_DIRECTION) {//If RIGHT is pressed
					this.direct = Player.RIGHT_DIRECTION;//Set the player 2 direction to right
				}
				
			}
			
			this.sok.close();//Close the socket
	}
	
    /**
     * Called on an object to free the memory or system ressources
     * used by this object.
     */
	public void finalize () {
		this.sok.finalize();//Finalize the socket
		super.finalize();//Call the superclass' finalize method
	}

}