/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.io.Keyboard;//Import the class moka.io.Keyboard
import moka.io.Graphics;//Import the class moka.io.Graphics
import moka.util.Timer;//Import the class moka.util.Timer

/**
 * This class is the main class of an object oriented network Tron game. Player 1 uses the arrows, player 2 its arrows on a remote calculator.
 * NB: The client program should be launched after the server one.
 *
 * @author  Frdric Brown
 */
public class Tron {
    /**
     * The main function is the entry point of a Moka program.
    */
    public static void main () {
    	Player[] p = new Player[2];//An array of Players.
    	Keyboard kb = new Keyboard();//A keyboard object used to read keys
    	short x_max;//The maximum x coordinate
    	short y_max;//The maximum y coordinate

		if (System.CALCULATOR == System.CALCULATOR_TI89) {//If the calculator is a TI-89
			x_max = 149;//Set the maximum x coordinate to 149
			y_max = 93;//Set the maximum y coordinate to 93
		}
		else {
			x_max = 239;//Set the maximum x coordinate to 239
			y_max = 122;//Set the maximum y coordinate to 122
		}

		System.clrscr();//Clear the screen
		System.println("TRON");//Write "TRON" then next line
		System.println("Player 1 uses the arrows,\nplayer 2 uses W, A, S and\nD. ESC to quit.\n\nPlayer 1 begins in upper\nleft, player 2 begins in\nlower right.");//Write the instructions then next line
		System.println();//Next line
		System.print("Press a key ...");//Write "Press a key ..."
		System.read();//Wait for the user to press a key

		p[0] = new Player1 ();//Add the player 1 to the array
		p[1] = new NetPlayer ();//Add the network player to the array (player 2)

		Keyboard.flush();//Flush the keyboard queue
    	while (!kb.key_ESC) {//While ESC is not pressed
    		System.clrscr();//Clear the screen
	    	System.println("TRON");//Write "TRON" then next line
	    	/*Write Player 1: (Number of points of player 1).*/
	    	System.println(p[0]);

	    	/*Write Player 1: (Number of points of player 2).*/
	    	System.println(p[1]);

	    	/*Draw the borders.*/
	    	Graphics.drawLine(0s, 30s, x_max, 30s, Graphics.NORMAL_ATTR);
	    	Graphics.drawLine(0s, 30s, 0s, y_max, Graphics.NORMAL_ATTR);
	    	Graphics.drawLine(0s, y_max, x_max, y_max, Graphics.NORMAL_ATTR);
	    	Graphics.drawLine(x_max, 30s, x_max, y_max, Graphics.NORMAL_ATTR);

			p[0].respawn(5s, 35s);//Prepares player 1 for a new round
			p[1].respawn((short)x_max - 5, (short)y_max - 5);//Prepares player 2 for a new round

			p[0].draw();//Draw player 1 at the starting location
			p[1].draw();//Draw player 1 at the starting location

			while ((p[0].getDirect() < 0 || p[1].getDirect() < 0) && !kb.key_ESC) {//While ESC key is not pressed and neither player 1 or player 2 has decided his starting direction
				kb.checkKeys();//Check which keys are pressed

				p[0].readKb(kb);//Call the player 1 keyboard reading routine
				p[1].readKb(kb);//Call the remote player keyboard reading routine
			}

			/*Game loop*/
	    	while (!kb.key_ESC) {//While ESC is not pressed
				kb.checkKeys();//Check which keys are pressed

				p[0].readKb(kb);//Read the keyboard to set the new direction of player 1
				p[1].readKb(kb);//Read the keyboard to set the new direction of player 2

				p[0].move();//Move player 1
				p[1].move();//Move player 2

				if (p[0].hit() && p[1].hit()) {//if the player 1 and the player 2 is about to hit a wall
					Graphics.draw((short)0, (short)0, "Nobody wins the round !", Graphics.REPLACE_ATTR);//Draw the String "Nobody wins the round !"
					Keyboard.flush();//Flush the keyboard queue
					System.read();//Wait for the user to press a key
					break;//break the game loop
				}
				else if (p[0].hit()) {//if the player 1 is about to hit a wall
					p[1].win();//Give a point to player 2 (increment p2p)
					Keyboard.flush();//Flush the keyboard queue
					System.read();//Wait for the user to press a key
					break;//break the game loop
				}
				else if (p[1].hit()) {//if the player 2 is about to hit a wall
					p[0].win();//Give a point to player 2 (increment p2p)
					Keyboard.flush();//Flush the keyboard queue
					System.read();//Wait for the user to press a key
					break;//break the game loop
				}

				p[0].draw();//draw a pixel on the new position of the player 1
				p[1].draw();//draw a pixel on the new position of the player 2

				Timer.wait(1);//Wait for an amount of time (1/20 second)
			}
	    }

		System.clrscr();//Clear the screen
		System.println("TRON");//Write "TRON" then next line
		if (p[0].getPoints() != p[1].getPoints()) {//If there is a winner
			System.print("Player ");//Write "Player "
			System.print((short)(p[0].getPoints() > p[1].getPoints() ? 1 : 2));//Write the winner number
		}
		else {
			System.print("Nobody");//Write "Nobody"
		}
		System.println(" wins the game !");//Write " wins !" then next line
		System.println();//Next line
		System.print("Press a key ...");//Write "Press a key ..."
		Keyboard.flush();//Flush the keyboard queue
		System.read();//Wait for the user to press a key

		p[0].finalize();//Finalize player 1
		p[1].finalize();//Finalize player 2
		kb.finalize();//Finalize kb
		System.free(p);//Finalize the players' array
    }
}