/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.x.Frame;
import moka.x.Button;
import moka.util.Timer;
import moka.lang.System;

/**
 * This class shows the use of Timer objects.
 *
 * @author  Frdric Brown
 */
public class TimerApp extends Frame {
	/** The quit button.*/
	private Button btnQuit;
	/** The timer.*/
	private Timer t;

	/** A message array */
	private String[] mess;
	/** The index of the message Array */
	private short ind;

	/**
	 * Constructs a new TimerApp frame.
	 */
	public TimerApp () {
		super("Publicity");//Call the superclass' constructor, passing the string "Publicity" as argument

		this.setBounds(0s, 0s, 150s, 100s);//Set the bounds of this frame to x=0, y=0, width=150 and height=100
		this.setMinimizeButtonVisible(false);//Hide the minimize button of the title bar
		this.defaultCloseOperation = Frame.EXIT_ON_CLOSE;//Set the default close operation for this frame to EXIT ON CLOSE

		this.btnQuit = new Button("Quit");//Instantiate a new Button labeled "Quit"
		this.btnQuit.setBounds(0s, 0s, 40s, 17s);//Set the bounds of the quit button to x=0, y=0, width=40 and height=17
		this.btnQuit.setMnemonic('Q');//Set the mnemonic for the quit button to 'Q'
		this.btnQuit.listen = this;//Set this frame as EventListener for the quit button
		this.add(this.btnQuit);//Add the quit button into this frame
		this.btnQuit.center();//Center the button

		this.t = new Timer(20);//Creates a new Timer that will trigger every second

		GEM.gem.register(this.t);//Register the timer to the GEM interrupt system.

		this.t.setActive(true);//Activate the timer

		this.t.listen = this; //Set this frame as the event listener of the timer

		this.mess = new String[3];//Create a new String array of size 3

		this.mess[0] = "Object oriented";
		this.mess[1] = "paradigm";
		this.mess[2] = "is great !";

		this.ind = 2;
	}

	/**
	 * Used to free the ressources used by this frame, is called when the frame is destroyed.
	 */
	public void finalize() {
		this.t.finalize();//Finalize the Timer

		/* Free the strings */
		this.mess[0].finalize();
		this.mess[1].finalize();
		this.mess[2].finalize();

		System.free(this.mess);//Free the array

		super.finalize();
	}

	/**
	 * Called when an Event occurs.
	 */
	public void eventTriggered (Object sender) {
		if (sender == this.btnQuit) {//If the sender is the quit button
			this.dispose();//Dispose of this window
		}
		if (sender == this.t) {
			this.ind = (this.ind + 1) % 3;
			this.setCaption(this.mess[this.ind].toString());//Show the next message in the title bar
		}
	}

	public static void main () {
		TimerApp ta = new TimerApp();//Instantiate a new TimerApp Frame
		ta.setVisible(true);//Show the TimerApp frame
	}
}